# decompyle3 version 3.3.2
# Python bytecode 3.7 (3394)
# Decompiled from: Python 3.8.1 (default, Jan  3 2020, 22:44:00) 
# [GCC 8.3.0]
# Embedded file name: ulang\parser\error.py
# Size of source mod 2**32: 82682 bytes


class SyntaxError(ValueError):

    def __init__(self, message, filename, lineno, colno, source=None):
        self.message_ = message
        self.filename_ = filename
        self.lineno_ = lineno if lineno > 0 else 1
        self.colno_ = colno if colno > 0 else 1
        self.source_ = source

    def __str__(self):
        msg = 'File "%s", line %d:%d, %s' % (
         self.filename_, self.lineno_, self.colno_, self.message_)
        if self.source_:
            line = self.source_[(self.lineno_ - 1)]
            col = ' ' * (self.colno_ - 1) + '^'
            msg = '%s\n%s\n%s' % (msg, line, col)
        return msg


# decompyle3 version 3.3.2
# Python bytecode 3.7 (3394)
# Decompiled from: Python 3.8.1 (default, Jan  3 2020, 22:44:00) 
# [GCC 8.3.0]
# Embedded file name: ulang\parser\lrparser.py
# Size of source mod 2**32: 82682 bytes
from rply.errors import ParsingError

class LRParser:

    def __init__(self, lrparser):
        self.lr_table = lrparser.lr_table
        self.error_handler = lrparser.error_handler

    def parse(self, tokenizer, state=None):
        from rply.token import Token
        lookahead = None
        lookaheadstack = []
        statestack = [0]
        symstack = [Token("$end", "$end")]

        current_state = 0
        while True:
            if self.lr_table.default_reductions[current_state]:
                t = self.lr_table.default_reductions[current_state]
                current_state = self._reduce_production(
                    t, symstack, statestack, state
                )
                continue

            if lookahead is None:
                if lookaheadstack:
                    lookahead = lookaheadstack.pop()
                else:
                    try:
                        lookahead = next(tokenizer)
                    except StopIteration:
                        lookahead = None

                if lookahead is None:
                    lookahead = Token("$end", "$end")

            ltype = lookahead.gettokentype()
            if ltype in self.lr_table.lr_action[current_state]:
                t = self.lr_table.lr_action[current_state][ltype]
                if t > 0:
                    statestack.append(t)
                    current_state = t
                    symstack.append(lookahead)
                    lookahead = None
                    continue
                elif t < 0:
                    current_state = self._reduce_production(
                        t, symstack, statestack, state
                    )
                    continue
                else:
                    n = symstack[-1]
                    return n
            elif self.error_handler is not None:
                if state is None:
                    self.error_handler(lookahead)
                else:
                    self.error_handler(state, lookahead)
                lookahead = None
                continue
            else:
                raise ParsingError(None, lookahead.getsourcepos())

    def _reduce_production(self, t, symstack, statestack, state):
        p = self.lr_table.grammar.productions[(-t)]
        pname = p.name
        plen = p.getlength()
        start = len(symstack) + (-plen - 1)
        assert start >= 0
        targ = symstack[start + 1:]
        start = len(symstack) + -plen
        if not start >= 0:
            raise AssertionError
        else:
            del symstack[start:]
            del statestack[start:]
            if state is None:
                value = p.func(targ)
            value = p.func(state, targ)
        symstack.append(value)
        current_state = self.lr_table.lr_goto[statestack[(-1)]][pname]
        statestack.append(current_state)
        return current_state


# decompyle3 version 3.3.2
# Python bytecode 3.7 (3394)
# Decompiled from: Python 3.8.1 (default, Jan  3 2020, 22:44:00) 
# [GCC 8.3.0]
# Embedded file name: ulang\runtime\repl.py
# Size of source mod 2**32: 82682 bytes
import sys, cmd
from ulang.parser.core import Parser
from ulang.parser.lexer import lexer
from ulang.runtime.env import create_globals

def is_close(源码):
    """
    Check if the given 源码 is closed,
    which means each '{' has a matched '}' 
    """
    关键词 = {
     'FUNC', 'OPERATOR', 'ATTR', 'TYPE',
     'FOR', 'LOOP', 'WHILE',
     'IF', 'ELIF', 'ELSE',
     'TRY', 'CATCH', 'FINALLY'}
    未配对之和 = 0
    unclosed_sum = 0
    if len(源码) > 1:
        if 源码[-2] == '\\':
            return False
        else:
            tokens = lexer.lex(源码)
            unclosed = []
            未配对 = [
            0, 0, 0]
            last = 2 * ['']
            for tok in tokens:
                c = tok.gettokentype()
                last[0], last[1] = last[1], c
                if c in 关键词:
                    unclosed.append(c)
                if c == 'LBRACE':
                    未配对[0] += 1
                elif c == 'RBRACE':
                    未配对[0] -= 1
                    if len(unclosed):
                        unclosed.pop(-1)
                elif c == '(':
                    未配对[1] += 1
                elif c == ')':
                    未配对[1] -= 1
                elif c == '[':
                    未配对[2] += 1
                elif c == ']':
                    未配对[2] -= 1
            未配对之和 = sum(未配对)
            unclosed_sum = len(unclosed)
            if unclosed_sum > 0:
                if 未配对_sum == 0:
                    if last[1] == 'NEWLINE':
                        if (last[0] == 'NEWLINE' or last[0]) == ';':
                            pass
                        return True
    return unclosed_sum == 0 and 未配对之和 == 0


def input_swallowing_interrupt(_input):

    def _input_swallowing_interrupt(*args):
        try:
            return _input(*args)
        except KeyboardInterrupt:
            print('^C')
            return '\n'

    return _input_swallowing_interrupt


class Repl(cmd.Cmd):
    """
    A simple wrapper for REPL using the python cmd module.
    """

    def __init__(self, ps1='> ', ps2='>> ', globals=None, locals=None):
        super().__init__()
        self.ps1 = ps1
        self.ps2 = ps2
        self.globals = globals
        self.locals = locals
        self.parser = Parser()
        self.prompt = ps1
        self.stmt = ''

    def do_help(self, arg):
        self.default('help(%s)' % arg)

    def do_quit(self, arg):
        self.default('quit(%s)' % arg)

    def do_EOF(self, arg):
        self.default('quit()')

    def onecmd(self, line):
        if line == 'EOF':
            return self.do_EOF(line)
        self.default(line)
        self.prompt = self.ps1 if len(self.stmt) == 0 else self.ps2

    def default(self, line):
        if line is not None:
            self.stmt += '%s\n' % line
            if not self.is_close():
                return
            try:
                try:
                    node = self.parser.parse('___=(%s);__print__(___)' % self.stmt, '<STDIN>')
                except Exception:
                    node = self.parser.parse(self.stmt, '<STDIN>')

                code = compile(node, '<STDIN>', 'exec')
                exec(code, self.globals, self.locals)
            except SystemExit:
                sys.exit()
            except BaseException as e:
                try:
                    sys.stderr.write('%s: %s\n' % (e.__class__.__name__, str(e)))
                finally:
                    e = None
                    del e

            finally:
                self.stmt = ''

    def is_close(self):
        return is_close(self.stmt)

    def cmdloop(self, *args, **kwargs):
        orig_input_func = cmd.__builtins__['input']
        cmd.__builtins__['input'] = input_swallowing_interrupt(orig_input_func)
        try:
            (super().cmdloop)(*args, **kwargs)
        finally:
            cmd.__builtins__['input'] = orig_input_func


def repl(ps1='> ', ps2='>> ', globals=None):
    """
    A simple read-eval-print-loop for the µLang program
    """
    info = [
     '\tglobals: to list all builtins',
     '\tquit: to quit the REPL',
     '\thelp: to show this message']
    if not globals:
        globals = create_globals(fname='<STDIN>')
    globals['globals'] = lambda : print('\n'.join([' %s (%s)' % (k, v.__class__.__name__) for k, v in globals.items() if k != '__builtins__' if k != '___']))
    globals['help'] = lambda *args: print('\n'.join(info)) if not args else print()
    Repl(ps1, ps2, globals).cmdloop("Welcome to ulang's REPL..\nType 'help' for more informations.")
    sys.exit(0)


import ast

class 语法树相关:

    def 格式化节点(节点, 层次):
        缩进 = "  "
        输出 = ""
        if isinstance(节点, list):
            输出 += "["
            for 子节点 in 节点:
                输出 += 语法树相关.格式化节点(子节点, 层次 + 1)
            输出 += "]"
        elif isinstance(节点, int):
            输出 += str(节点)
        elif isinstance(节点, str):
            输出 += "'" + 节点 + "'"
        else:
            输出 += type(节点).__name__ + "("
            属性个数 = 0
            for 属性 in ast.iter_fields(节点):
                属性个数 += 1
                输出 += "\n" + 缩进 * 层次 + 属性[0] + "="
                输出 += 语法树相关.格式化节点(属性[1], 层次 + 1)
            if isinstance(节点, ast.stmt) or isinstance(节点, ast.expr):
                输出 += "\n" + 缩进 * 层次 + "lineno=" + str(节点.lineno)
                输出 += "\n" + 缩进 * 层次 + "col_offset=" + str(节点.col_offset)
            if 属性个数 == 0:
                return 输出 + ")"
            return 输出 + "\n" + 缩进 * (层次 - 1) + ")"
        return 输出